# import modules
# load modules
# Note: the Bem2dlib.pyx file in the bem_lib folder must be compiled usoing cython and cython-gsl
# The script is only tested on Linux machines
# This example show the calculation using the 2-dimensinal BEM routine based on a 4 pol with circular rods
import os
import time
import sys
from multiprocessing import Pool
from numpy import linalg as LA
import numpy as np
from bem_lib import Bem2dlib
from bem_lib import read_msh_file
from bem_lib import calc_multipol_coeff_2d


start1 = time.time()
#################################################################
# define constants

fac_R =  sys.argv[1]
filename = '4_pol_' + str(fac_R) # filename
r0 = 5.0  # distance to the electrodes [m]
phiCount = 361  # number of point for calculating multipolcoeff

#################################################################

#  read in msh file
meshname = 'meshfiles/4_pol_'+str(fac_R)+'.msh'
read_msh_file.read_mesh(meshname)
# load mapping of linear elements
pts = np.loadtxt('coor.txt',usecols=(1, 2))
pts1 = np.loadtxt('mapping.txt',usecols=(0,1),dtype=(int))
boundc = np.loadtxt('mapping.txt',usecols=(1,2),dtype=(int))
voltage = []

for j in range(len(boundc[:,1])):
    if boundc[j,1]%2==0:
        voltage.append((1.))
    else:
        voltage.append((-1.))

n = int(len(voltage))

# check if the number of boundary elements are right ##############
if len(boundc[:,1]) == n:
    print 'number of boundary elements = ', n
else:
    print 'error: number of dirichlet data is wrong!'
    sys.exit()
###################################################################
# define andcalculate the single layer operator on boundary
def G(i,j):
    if i != j:
        return -.25*Bem2dlib.int_S_func(i,j,pts1,pts)
    else:
        return -.25*Bem2dlib.int_S_func_s(i,j,pts1,pts)


Gfunc = np.vectorize(G)
def s_layer(s):
    return Gfunc(np.arange(len(voltage)),s)

start = time.time()
if __name__ == "__main__":
    po = Pool()
    resG = po.map(s_layer,((s) for s in xrange(len(voltage))))

stop = time.time()
print 'time for collecting Single Layer OP =',stop-start,'s'
####################################################################

potentialvector = voltage
v = np.array([potentialvector]).T

print '#calculate solution on gamma'
start = time.time()

dh = LA.solve(resG, -v)
solution = []

for i in range(n):
    solution.append((.5*(pts[pts1[i,1]][0]+pts[pts1[i,0]][0]),.5*(pts[pts1[i,1]][1]+pts[pts1[i,0]][1]),dh[i]))
           
np.savetxt('chargedist'+filename+'_num_of_elements'+str(n)+'.txt', solution)
stopp = time.time()

print 'Solution Time Charge ditribution =',stopp-start,'s'
####################################################################
# plot neuman trace on gamma

import matplotlib as mpl
from mpl_toolkits.mplot3d import Axes3D
import matplotlib.pyplot as plt
mpl.rcParams['legend.fontsize'] = 10
fig = plt.figure()
ax = fig.gca(projection='3d')
solfun = np.loadtxt('chargedist'+filename+'_num_of_elements'+str(n)+'.txt')
x = solfun[:, 0]
y = solfun[:, 1]
neuman_tr = solfun[:, 2]
ax.plot( x,y, neuman_tr, 'x', label='sol_fun')

ax.set_xlabel('X direction')
ax.set_ylabel('Y direction')
ax.set_zlabel('neuman_trace')

ax.legend()
plt.show()

###############################################################
#calculating evaluating points
phi = np.mgrid[0:2*np.pi:phiCount*1j]   
R = r0*.8
x = R * np.cos(phi)
y = R * np.sin(phi)

pointxy = []
for i in xrange(phiCount):
    pointxy.append((x[i], y[i]))
        
np.savetxt('evalpoints.txt',pointxy)
ptx = np.loadtxt('evalpoints.txt')

################################################################
# calculate singlelayer integrals for solution electrical potential at evaluation points
def G_pot(i,j):
    return -.25*Bem2dlib.int_S_func_ev(i,j,pts1,pts,ptx)
    
start = time.time()
G_efunc = np.vectorize(G_pot)
def s_layer_pot(s):
    return G_efunc(np.arange(len(voltage)),s)

if __name__ == "__main__":
    po = Pool()
    resG_pot = po.map(s_layer_pot,((s) for s in xrange(len(ptx[:,0]))))
        
stop = time.time()
print 'time for collecting Single Layer OP for potential at evaluation points =',stop-start,'s'
################################################################
# calculate potential at evaluation points 
start = time.time()

print '#calculate Potential'
potential = []
for g in xrange(len(ptx[:,0])):
    pot = np.dot(resG_pot[g],dh)
    potential.append((ptx[g,0],ptx[g,1],pot))

savename = 'field_'+filename+'_num_of_elements'+str(n)+'.txt'
savename_mp = 'field_'+filename+'.txt'
np.savetxt(savename, potential)

stopp = time.time()

print 'time for calculating potential al evaluation points =',stopp-start, 's'

##################################################################
#claculate multipolcoeffizients
calc_multipol_coeff_2d.multipolceff(phiCount, R, r0, 22, savename, n,2,True,savename_mp)
stop1 = time.time()
print 'time for complete calculating =',stop1-start1, 's'
# remove 
import os
os.remove('coor.txt')
os.remove('mapping.txt')
os.remove('evalpoints.txt')
os.remove('mapp.txt')

