#!/usr/bin/python
# import all modules
# Note the Bem2d_axislib.pyx file must be compiled using cython and cython-gsl in order to run the programm
# Note. The programm is only running on Linux maschines because of the parallelization of the code
# To calcuulate the Multipolecoeffitients use the multipol_coeff_calc.py script
import os 
import numpy as np
import sys
import string
from multiprocessing import Pool
import time
from scipy.sparse import coo_matrix
from numpy import linalg as LA
from scipy.integrate import quad
import matplotlib.pyplot as plt
import matplotlib as mpl
from mpl_toolkits.mplot3d import Axes3D
import Bem2d_axislib



###############################################################################
#load the example mesh
###############################################################################

fobj = open('tim_esser_sret.msh')

liste_mesh = []
for i in fobj:
    liste_mesh.append(i.rstrip())
   
n_1 = liste_mesh[4]

n_1 = int(n_1)
n  = liste_mesh[n_1+7]
num_bound_elements = int(n)


# convert list of boundary coordinates in array
koor_lst = []
for i in range(5,n_1+5):
    s = liste_mesh[i]
    koor_lst.append([float(x) for x in s.split()])

koor = np.array(koor_lst)

zourd_lst = []
for i in range(n_1+8,n_1+8+num_bound_elements):
    s = liste_mesh[i]
    zourd_lst.append([int(x) for x in s.split()])

zuord = np.array(zourd_lst)

listemap = []
for i in range(num_bound_elements):
    listemap.append((zuord[i,5]-1,zuord[i,6]-1,zuord[i,3]))

pts = koor[:,1:3]
pts1 = np.array(listemap)[:,0:2]
bvp = np.array(listemap)[:,2]
###############################################################################

# numbering of electrodes and corresponding volatges ##########################
elec_num = np.array([43,44,45,46,47,48])
dirichlet = np.array([.5,.5,-.5,-.5,0,0])
num_elec = len(elec_num)

################################
# voltage i-te electrode
################################
voltage = []
for k in range(num_elec):
    for i in range(len(pts1[:,0])):
        if bvp[i] == elec_num[k]:
            voltage.append(dirichlet[k])
        else:
            pass



# start calculation of Singlelayer for charge distribution on boundary
def G(i,j):
    if i != j:
        return Bem2d_axislib.int_S_func(i,j,pts1,pts)
    else:
        return Bem2d_axislib.int_S_func_s(i,j,pts1,pts)


start = time.time()
Gfunc = np.vectorize(G)

def s_layer(s):
    return Gfunc(np.arange(len(voltage)),s)

if __name__ == "__main__":
    po = Pool()
    resG = po.map(s_layer,((s) for s in xrange(len(voltage))))
    np.savetxt('Gel.txt', resG)


dataG = np.loadtxt('Gel.txt')

G_el = coo_matrix(([],([],[])),shape=(num_bound_elements,num_bound_elements)).todense()

for p in xrange(num_bound_elements):
    for s in xrange(num_bound_elements):
        G_el[p,s]= dataG[p,s]
stop = time.time()
print 'time for collecting Single Layer OP =',stop-start,'s'
####################################################################################
potentialvector = voltage
v = np.array([potentialvector]).T
###################################################################################
print '#calculate solution on gamma'
dh = LA.solve(G_el, v)
print dh

solution = []
for i in range(num_bound_elements):
    solution.append((.5*(pts[pts1[i,1]][0]+pts[pts1[i,0]][0]),.5*(pts[pts1[i,1]][1]+pts[pts1[i,0]][1]),dh[i]))
    
   
np.savetxt('chargedist.txt', solution)
stopp = time.time()

print 'Time for calculation of neumann trace on boundary =',stopp-start, 's'

##plot charge distribution #####################################
mpl.rcParams['legend.fontsize'] = 10
fig = plt.figure()
ax = fig.gca(projection='3d')
data = np.loadtxt('chargedist.txt')
x = data[:,0]
y = data[:,1]
z = data[:,2]
ax.scatter(x, y, z); 
plt.show()

########################################calc evaluation points for multipüole expansion ##################################

R = 3.0
data1 = np.loadtxt('gauss_n_w_31.txt')
theta = (data1[:,0]) 
y = R*theta
x = R*np.sin(np.arccos(theta))

evaluationPoints = np.vstack((x.ravel(), y.ravel()))
pointxy = []
for i in xrange(len(theta)):
    pointxy.append((x[i], y[i]))

np.savetxt('evalpoints.txt',pointxy)
ptx = np.loadtxt('evalpoints.txt')

################################################################################

################################################################################
# calculate E_field r
def G_ev_dr(i,j):
    return Bem2d_axislib.int_S_func_ev_dr(i,j,pts1,pts,ptx)
      
start = time.time()
Gfunc_ev_dr = np.vectorize(G_ev_dr)

def s_layer_ev_dr(s):
    return Gfunc_ev_dr(np.arange(len(voltage)),s)

if __name__ == "__main__":
    po = Pool()
    resG_ev_dr = po.map(s_layer_ev_dr,((s) for s in xrange(len(ptx[:,0]))))
    np.savetxt('G_e_field_dr.txt', resG_ev_dr)

data_e_field_dr = np.loadtxt('G_e_field_dr.txt')

G_e_field_dr = coo_matrix(([],([],[])),shape=(len(ptx[:,0]),n)).todense()

for p in xrange(len(ptx[:,0])):
    for s in xrange(num_bound_elements):
        G_e_field_dr[p,s]= data_e_field_dr[p,s]
stop = time.time()
print 'time for collecting Single Layer OP for Electric Field in r-direction =',stop-start,'s'
######################################################################################
# calculate E_field z
def G_ev_dz(i,j):
    return Bem2d_axislib.int_S_func_ev_dz(i,j,pts1,pts,ptx)
      
start = time.time()
Gfunc_ev_dz = np.vectorize(G_ev_dz)

def s_layer_ev_dz(s):
    return Gfunc_ev_dz(np.arange(len(voltage)),s)

if __name__ == "__main__":
    po = Pool()
    resG_ev_dz = po.map(s_layer_ev_dz,((s) for s in xrange(len(ptx[:,0]))))
    np.savetxt('G_e_field_dz.txt', resG_ev_dz)

data_e_field_dz = np.loadtxt('G_e_field_dz.txt')

G_e_field_dz = coo_matrix(([],([],[])),shape=(len(ptx[:,0]),n)).todense()

for p in xrange(len(ptx[:,0])):
    for s in xrange(num_bound_elements):
        G_e_field_dz[p,s]= data_e_field_dz[p,s]
stop = time.time()
print 'time for collecting Single Layer OP for Electric Field in z-direction =',stop-start,'s'
######################################################################################
# calculate potential
def G_pot_ev(i,j):
    return Bem2d_axislib.int_S_func_ev(i,j,pts1,pts,ptx)
      
start = time.time()
Gfunc_pot_ev = np.vectorize(G_pot_ev)

def s_layer_pot_ev(s):
    return Gfunc_pot_ev(np.arange(len(voltage)),s)

if __name__ == "__main__":
    po = Pool()
    resG_pot_ev = po.map(s_layer_pot_ev,((s) for s in xrange(len(ptx[:,0]))))
    np.savetxt('G_pot_ev.txt', resG_pot_ev)

data_pot_ev = np.loadtxt('G_pot_ev.txt')

G_pot_ev = coo_matrix(([],([],[])),shape=(len(ptx[:,0]),n)).todense()

for p in xrange(len(ptx[:,0])):
    for s in xrange(num_bound_elements):
        G_pot_ev[p,s]= data_pot_ev[p,s]
stop = time.time()
print 'time for collecting Single Layer OP for electrical potential=',stop-start,'s'

### calculate the solution #####
dh1 = np.loadtxt('chargedist.txt',usecols=(1,2))
dh2 = dh1[:,1]
print 'calculate the solution E_r, E_z, Potential'
potential_out = []
count = 0
for g in xrange(len(ptx[:,0])):
    evalpot_dr = (np.dot(G_e_field_dr[g,:],dh2)) 
    evalpot_dz = (np.dot(G_e_field_dz[g,:],dh2))
    evalpot = (np.dot(G_pot_ev[g,:],dh2))
    potential_out.append((ptx[g,0],ptx[g,1],evalpot_dr,evalpot_dz,evalpot))

### save solution and relevant stuff ####
np.savetxt('field_num_of_elem_'+str(n)+'.txt', potential_out)


stopp = time.time()

print 'calculation Time for E_r, E_z, Potential =',stopp-start, 's'

# remove old files from folder
os.remove('G_pot_ev.txt')
os.remove('G_e_field_dz.txt')
os.remove('G_e_field_dr.txt')
os.remove('Gel.txt')


